@@ -123,7 +123,7 @@ requires proper credentials.")
                     urllib.parse.quote_plus(key.encode("utf-8")),
                     urllib.parse.quote_plus(value.encode("utf-8"))
                 ) for key, value in
-                list(data.items())
+                data.items()
             ])
         else:
             # Otherwise, we can just use the vanilla urllib prep method
@@ -189,44 +189,76 @@ class DocumentClient(BaseDocumentCloudClient):
         except:
             return False
 
-    def _get_search_page(self, query, page, per_page):
+    def _get_search_page(
+        self,
+        query,
+        page,
+        per_page=1000,
+        mentions=3,
+        data=False,
+    ):
         """
         Retrieve one page of search results from the DocumentCloud API.
         """
+        if mentions > 10:
+            raise ValueError("You cannot search for more than 10 mentions")
         params = {
             'q': query,
             'page': page,
             'per_page': per_page,
-            'mentions': 3,
+            'mentions': mentions,
         }
-        data = self.fetch('search.json', params)
-        return data.get("documents")
+        if data:
+            params['data'] = 'true'
+        response = self.fetch('search.json', params)
+        return response.get("documents")
 
-    def search(self, query):
+    def search(self, query, page=None, per_page=1000, mentions=3, data=False):
         """
         Retrieve all objects that make a search query.
 
+        Will loop through all pages that match unless you provide
+        the number of pages you'd like to restrict the search to.
+
         Example usage:
 
             >> documentcloud.documents.search('salazar')
-
-        Based on code by Chris Amico
         """
-        page = 1
-        document_list = []
-        # Loop through all the search pages and fetch everything
-        while True:
-            results = self._get_search_page(query, page=page, per_page=1000)
-            if results:
-                document_list += results
-                page += 1
-            else:
-                break
+        # If the user provides a page, search it and stop there
+        if page:
+            document_list = self._get_search_page(
+                query,
+                page=page,
+                per_page=per_page,
+                mentions=mentions,
+                data=data,
+            )
+        # If the user doesn't provide a page keep looping until you have
+        # everything
+        else:
+            page = 1
+            document_list = []
+            # Loop through all the search pages and fetch everything
+            while True:
+                results = self._get_search_page(
+                    query,
+                    page=page,
+                    per_page=per_page,
+                    mentions=mentions,
+                    data=data,
+                )
+                if results:
+                    document_list += results
+                    page += 1
+                else:
+                    break
+        # Convert the JSON objects from the API into Python objects
         obj_list = []
         for doc in document_list:
             doc['_connection'] = self._connection
             obj = Document(doc)
             obj_list.append(obj)
+        # Pass it back out
         return obj_list
 
     def get(self, id):
@@ -565,6 +597,39 @@ class Annotation(BaseAPIObject):
     location = property(get_location)
 
 
+class DocumentDataDict(dict):
+    """
+    The key/value store DocumentCloud allows with each Document.
+
+    Functions the same as a standard Python dictionary, with two exception:
+    key names protected by DocumentCloud are restricted and only strings
+    are allowed.
+
+    If you try to set an integer or other type as a key or value it will
+    throw a TypeError.
+    """
+    def __init__(self, *args, **kwargs):
+        for d in args:
+            for key, value in d.items():
+                self.validate_key(key)
+                self.validate_value(value)
+        dict.__init__(self, *args, **kwargs)
+
+    def __setitem__(self, key, value):
+        self.validate_key(key)
+        self.validate_value(value)
+        dict.__setitem__(self, key, value)
+
+    def validate_key(self, key):
+        is_valid_data_keyword(key)
+        if not isinstance(key, six.string_types):
+            raise TypeError("data attribute keys must be strings")
+
+    def validate_value(self, value):
+        if not isinstance(value, six.string_types):
+            raise TypeError("data attribute values must be strings")
+
+
 class Document(BaseAPIObject):
     """
     A document returned by the API.
@@ -669,21 +734,18 @@ class Document(BaseAPIObject):
         # Make sure a dict got passed it
         if not isinstance(data, type({})):
             raise TypeError("This attribute must be a dictionary.")
-        # Validate keywords
-        for keyword in list(data.keys()):
-            is_valid_data_keyword(keyword)
         # Set the attribute
-        self.__dict__['data'] = data
+        self.__dict__['data'] = DocumentDataDict(data)
 
     def get_data(self):
         """
         Fetch the data field if it does not exist.
         """
         try:
-            return self.__dict__['data']
+            return DocumentDataDict(self.__dict__['data'])
         except KeyError:
             self._lazy_load()
-            return self.__dict__['data']
+            return DocumentDataDict(self.__dict__['data'])
     data = property(get_data, set_data)
 
     def get_annotations(self):
@@ -738,6 +800,7 @@ class Document(BaseAPIObject):
 
     def _get_url(self, url):
         if self.access == 'public':
+            url = url.replace("https://", "http://")
             req = urllib.request.Request(url)
             try:
                 return urllib.request.urlopen(req).read()
